/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::RobotHealthDummy
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/services/tasks/RunningTask.h>
#include <ArmarXCore/core/time/TimeUtil.h>

#include <RobotAPI/interface/components/RobotHealthInterface.h>

namespace armarx
{
    /**
     * @class RobotHealthDummyPropertyDefinitions
     * @brief
     */
    class RobotHealthDummyPropertyDefinitions : public armarx::ComponentPropertyDefinitions
    {
    public:
        RobotHealthDummyPropertyDefinitions(std::string prefix) :
            armarx::ComponentPropertyDefinitions(prefix)
        {
            defineOptionalProperty<std::string>(
                "RobotHealthTopicName", "RobotHealthTopic", "Name of the RobotHealth topic");
            defineOptionalProperty<std::string>(
                "SleepMode",
                "nanosleep",
                "Which sleep function to use: nanosleep, sleepwait, yieldwait, busywait");

            //defineRequiredProperty<std::string>("PropertyName", "Description");
            //defineOptionalProperty<std::string>("PropertyName", "DefaultValue", "Description");
        }
    };

    /**
     * @defgroup Component-RobotHealthDummy RobotHealthDummy
     * @ingroup RobotAPI-Components
     * A description of the component RobotHealthDummy.
     *
     * @class RobotHealthDummy
     * @ingroup Component-RobotHealthDummy
     * @brief Brief description of class RobotHealthDummy.
     *
     * Detailed description of class RobotHealthDummy.
     */
    class RobotHealthDummy : virtual public armarx::Component
    {
    public:
        /**
         * @see armarx::ManagedIceObject::getDefaultName()
         */
        std::string
        getDefaultName() const override
        {
            return "RobotHealthDummy";
        }

        int NanoSleep(long nanosec);
        void yieldwait(long microseconds);
        void busywait(long microseconds);
        void sleepwait(long microseconds);

    protected:
        /**
         * @see armarx::ManagedIceObject::onInitComponent()
         */
        void onInitComponent() override;

        /**
         * @see armarx::ManagedIceObject::onConnectComponent()
         */
        void onConnectComponent() override;

        /**
         * @see armarx::ManagedIceObject::onDisconnectComponent()
         */
        void onDisconnectComponent() override;

        /**
         * @see armarx::ManagedIceObject::onExitComponent()
         */
        void onExitComponent() override;

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void runTask();

        RobotHealthInterfacePrx robotHealthTopicPrx;
        RunningTask<RobotHealthDummy>::pointer_type dummyTask;

        std::string sleepmode;
    };
} // namespace armarx
