
#include "SkillProviderExample.h"

namespace armarx::skills::provider
{
    SkillProviderExample::SkillProviderExample() : SkillProviderComponentPluginUser()
    {
    }

    armarx::PropertyDefinitionsPtr
    SkillProviderExample::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());
        return defs;
    }

    std::string
    SkillProviderExample::getDefaultName() const
    {
        return "SkillProviderExample";
    }

    void
    SkillProviderExample::onInitComponent()
    {
        // Add example skill
        ARMARX_INFO << "Adding skill HelloWorldSkill";
        addSkillFactory<HelloWorldSkill>();

        // Add another lambda example skill
        {
            skills::SkillDescription fooDesc{.skillId = SkillID{.skillName = "Foo"},
                                             .description = "This skill does exactly nothing.",
                                             .timeout =
                                                 armarx::core::time::Duration::MilliSeconds(1000)};
            addSkillFactory(fooDesc,
                            []()
                            {
                                std::cout << "Hello from Foo." << std::endl;
                                return TerminatedSkillStatus::Succeeded;
                            });
        }

        // Add another example skill
        ARMARX_INFO << "Adding skill CallbackSkill";
        addSkillFactory<CallbackSkill>();

        // Add timeout skill
        ARMARX_INFO << "Adding skill TimeoutSkill";
        addSkillFactory<TimeoutSkill>();

        // chaining
        ARMARX_INFO << "Adding skill ChainingSkill";
        addSkillFactory<ChainingSkill>();

        // incomplete and prepare
        ARMARX_INFO << "Adding skill IncompleteSkill";
        addSkillFactory<IncompleteSkill>();
    }

    void
    SkillProviderExample::onConnectComponent()
    {
    }

    void
    SkillProviderExample::onDisconnectComponent()
    {
    }

    void
    SkillProviderExample::onExitComponent()
    {
    }
} // namespace armarx::skills::provider
