#pragma once

#include <stack>

#include <QTreeWidget>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>

#include "AronTreeWidgetItem.h"
#include "Data.h"
#include "visitors/AronTreeWidgetCreator.h"

namespace armarx
{
    // Main controller for any AronTreeWidget GUI. It attaches itself to the parent and needs the active widget.
    // It's responsible to handle all signals for non-widget fields and click events.
    class AronTreeWidgetController : public QObject
    {
        Q_OBJECT
    public:
        AronTreeWidgetController(QTreeWidget* tree,
                                 QTreeWidgetItem* parent,
                                 const aron::type::ObjectPtr& type,
                                 const aron::data::DictPtr& data = nullptr);

        aron::data::DictPtr convertToAron() const;
        void setFromAron(const aron::data::DictPtr&);

    private:
        QTreeWidgetItem* parent;
        QTreeWidget* tree;

        aron::type::ObjectPtr type;

    private slots:
        // allows most primitive fields to be directly editable in the tree
        // only String will open up a new widget
        void onTreeWidgetItemDoubleClicked(QTreeWidgetItem* item, int column);
        // check the new user input. Maybe undo if the field must not be edited.
        // Also highlight if the input cannot be parsed. (Or the errors are now fixed)
        void onTreeWidgetItemChanged(QTreeWidgetItem* item, int column);

    public slots:
        // hook for items to show a context menu (add  / delete element)
        void ShowContextMenu(const QPoint&);
    };

    using AronTreeWidgetControllerPtr = std::shared_ptr<AronTreeWidgetController>;
} // namespace armarx
