/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <ArmarXCore/interface/core/PackagePath.ice>

#include <RobotAPI/interface/armem/mns.ice>
#include <RobotAPI/interface/skills/SkillManagerInterface.ice>
#include <RobotAPI/interface/units/ForceTorqueUnit.ice>
#include <RobotAPI/interface/units/HandUnitInterface.ice>
#include <RobotAPI/interface/units/KinematicUnitInterface.ice>
#include <RobotAPI/interface/units/LocalizationUnitInterface.ice>
#include <RobotAPI/interface/units/PlatformUnitInterface.ice>
#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.ice>

module armarx
{
    module robot_name_service
    {
        module dto
        {

            module location
            {
                enum Side
                {
                    LEFT,
                    RIGHT,
                    UNKNOWN
                };
            };

            struct TCPInfo
            {
                string name;
                string ft_name;
                location::Side side;
                HandUnitInterface* handUnitInterface;
            };

            // maps tcp name to tcp
            dictionary<string, TCPInfo> TCPInfos;

            struct ArmInfo
            {
                string kinematicChainName;
                location::Side side;
                TCPInfo tcpInfo;
            };

            // maps kinematic chain name to arm
            dictionary<string, ArmInfo> ArmInfos;

            struct RobotInfo
            {
                string name;
                armarx::data::PackagePath xmlPackagePath;

                // Memory and skills. MAY BE NULL
                armarx::armem::mns::MemoryNameSystemInterface* memoryNameSystem;
                armarx::skills::manager::dti::SkillManagerInterface* skillManager;

                // kinematic stuff. MAY BE EMPTY
                ArmInfos armInfos;

                // RobotUnit
                armarx::RobotUnitInterface* robotUnit;

                // TODO: Feel free to extend!
            };
        };

        module dti
        {
            interface RobotNameServiceInterface
            {
                bool registerRobotInfo(dto::RobotInfo robot);
                void unregisterRobotInfo(string name);
                optional(1) dto::RobotInfo getRobotInfo(string name);
            };
        };
    };
};
