#pragma once

#include <filesystem>
#include <map>
#include <string>
#include <vector>

#include <RobotAPI/libraries/armem/core/MemoryID.h>

namespace armarx::armem::server::ltm::util::fs
{
    namespace detail
    {
        inline const std::map<std::string, std::string> EscapeTable = {{"/", "|"}};

        std::string escapeName(const std::string& segmentName);

        std::string unescapeName(const std::string& escapedName);

        std::string toDayString(const Time& t);

        std::string toSecondsString(const Time& t);

        bool isNumberString(const std::string& s);

        bool isDateString(const std::string& s);
    } // namespace detail

    std::filesystem::path toPath(const std::filesystem::path& base, const armem::MemoryID& id);

    bool directoryExists(const std::filesystem::path& p);

    bool fileExists(const std::filesystem::path& p);

    void ensureDirectoryExists(const std::filesystem::path& p, bool createIfNotExistent = false);

    void ensureFileExists(const std::filesystem::path& p, bool createIfNotExistent = false);

    void writeDataToFile(const std::filesystem::path& p, const std::vector<unsigned char>& data);

    std::vector<unsigned char> readDataFromFile(const std::filesystem::path& p);

    std::vector<std::filesystem::path> getAllDirectories(const std::filesystem::path& p);

    std::vector<std::filesystem::path> getAllFiles(const std::filesystem::path& p);

} // namespace armarx::armem::server::ltm::util::fs
