#pragma once

#include <map>
#include <memory>
#include <string>

#include <SimoxUtility/json.h>

#include <RobotAPI/libraries/armem/core/MemoryID.h>

#include <bsoncxx/builder/stream/array.hpp>
#include <bsoncxx/builder/stream/document.hpp>
#include <bsoncxx/builder/stream/helpers.hpp>
#include <bsoncxx/json.hpp>
#include <mongocxx/client.hpp>
#include <mongocxx/instance.hpp>
#include <mongocxx/pool.hpp>
#include <mongocxx/stdx.hpp>
#include <mongocxx/uri.hpp>

namespace armarx::armem::server::ltm::util::mongodb
{

    namespace detail
    {
        inline const std::map<std::string, std::string> EscapeTable = {{"/", "|"}};

        std::string escapeName(const std::string& segmentName);

        std::string unescapeName(const std::string& escapedName);

        bool insert(mongocxx::collection& coll, const nlohmann::json& value);

        std::optional<nlohmann::json> contains(mongocxx::collection& coll,
                                               const nlohmann::json& value);

        bool update(mongocxx::collection& coll,
                    const nlohmann::json& query,
                    const std::string& vkey,
                    const nlohmann::json& vdata);
    } // namespace detail

    std::string toDocumentID(const armem::MemoryID&);
    std::string toCollectionName(const armem::MemoryID&);

    std::optional<mongocxx::database> databaseExists(mongocxx::client&,
                                                     const std::string& databaseName);
    mongocxx::database ensureDatabaseExists(mongocxx::client&,
                                            const std::string& databaseName,
                                            bool createIfNotExistent);

    std::optional<mongocxx::collection> collectionExists(mongocxx::database&,
                                                         const std::string& collectionName);
    mongocxx::collection ensureCollectionExists(mongocxx::database&,
                                                const std::string& collectionName,
                                                bool createIfNotExistent);

    std::optional<nlohmann::json> documentExists(mongocxx::collection& collection,
                                                 const nlohmann::json& query);

    nlohmann::json ensureDocumentExists(mongocxx::collection& collection,
                                        const nlohmann::json& query,
                                        bool createIfNotExistent = false);

    nlohmann::json readDataFromDocument(mongocxx::collection& collection,
                                        const nlohmann::json& query);

    void writeDataToDocument(mongocxx::collection& collection,
                             const nlohmann::json& query,
                             const nlohmann::json& data);

    std::vector<nlohmann::json> getAllDocuments(mongocxx::collection& collection);

} // namespace armarx::armem::server::ltm::util::mongodb
