#pragma once

#include <string>
#include <vector>

#include <ArmarXCore/core/time/Duration.h>

#include <RobotAPI/interface/skills/SkillProviderInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>
#include <RobotAPI/libraries/aron/core/type/variant/container/Object.h>

#include "SkillID.h"

namespace armarx
{
    namespace skills
    {
        struct SkillDescription
        {
            SkillID skillId;
            std::string description = "";
            aron::data::DictPtr rootProfileDefaults = nullptr;
            armarx::core::time::Duration timeout = armarx::core::time::Duration::MilliSeconds(-1);
            aron::type::ObjectPtr parametersType = nullptr;
            aron::type::ObjectPtr resultType = nullptr;

            provider::dto::SkillDescription toProviderIce() const;
            manager::dto::SkillDescription toManagerIce() const;

            static SkillDescription FromIce(const provider::dto::SkillDescription& i,
                                            const std::optional<ProviderID>& = std::nullopt);
            static SkillDescription FromIce(const manager::dto::SkillDescription& i);
        };

        template <class T>
        concept isSkillDescription = std::is_base_of<SkillDescription, T>::value;
    } // namespace skills
} // namespace armarx
